"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  var REPLACED = Symbol();
  var bluebird = t.identifier("Promise");
  var funcNode = null;

  return {
    visitor: {

      FunctionExpression: {
        enter: function enter(path) {
          var node = path.node;

          if (!node[REPLACED] && node.async === true) {
            funcNode = node;
          }
        },
        exit: function exit(path) {
          if (funcNode && !funcNode[REPLACED] && funcNode.async === true) {
            // generators use 'coroutine' helper, other wise just 'method'
            var methodName = funcNode.generator ? "coroutine" : "method";
            var asyncWrap = t.memberExpression(bluebird, t.identifier(methodName));

            funcNode[REPLACED] = true;
            funcNode.async = false; // prevent recursive visits
            path.replaceWith(t.callExpression(asyncWrap, [funcNode]));

            funcNode = null;
          }
        }
      },

      AwaitExpression: function AwaitExpression(path) {
        var node = path.node;

        var arg = node.argument;
        path.replaceWith(t.yieldExpression(arg));

        // if there's an 'await', we'll turn the function into a generator
        if (funcNode) {
          // assert that 'await' is used in async methods only
          t.assertFunctionExpression(funcNode, { async: true });

          funcNode.generator = true;
        }
      }
    }
  };
};